/*
 * Complex Bessel fuction test program
 *
 * This program computes following Kelvin Function Tables.
 *  H.B.Dwight,- TABLES OF INTEGRALS AND OTHER MATHEMATICAL DATA Table 1050
 *       (MACMILLAN PUBLISHING CO.,INC.)
 *
 * 2016-09-10 Kouichi Hirabayashi
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <complex.h>
#include "Cbes.h"

int berbei(void);
int kerkei(void);
int bernbein(void);
int kernkein(void);

COMPLEX I2N(int);

int main(void)
{
  berbei(); printf("\n");
  kerkei(); printf("\n");
  bernbein();
  kernkein();

  exit(0);
}

// ber(x) + j*bei(x) = I0(x*sqrt(-1))
// ber'(x) = (d/dx)ber(x)

int berbei()
{
  double x;
  COMPLEX z, z1, z2;

  printf("%4s %13s %13s %13s %13s\n",
	"x", "ber(x)", "bei(x)", "ber'(x)", "bei'(x)");
  for (x = 0.0; x <= 10.0; x += 0.1) {
	z = csqrt(I) * x;
	z1 = In(0, z);
	z2 = csqrt(I) * Ind(0, z);
	printf("%4.1f %13.9f %13.9f %13.9f %13.9f\n",
		x, creal(z1), cimag(z1), creal(z2), cimag(z2));
  }
  return 0;
}

// ker(x) + j*kei(x) = K0(x*sqrt(-1))
// ker'(x) = (d/dx)ker(x)

int kerkei()
{
  double x;
  COMPLEX j, z, z1, z2;

  printf("%4s %13s %13s %13s %13s\n",
	"x", "ker(x)", "kei(x)", "ker'(x)", "kei'(x)");
  for (x = 0.0; x <= 10.0; x += 0.1) {
	z = csqrt(I) * x;
	if (x == 0.0) {
		z1 = HUGE_VAL - 0.25 * atan2(0.0, -1.0) * I;
		z2 = -HUGE_VAL - 0.0 * I;
	}
	else {
		z1 = Kn(0, z);
		z2 = csqrt(I) * Knd(0, z);
	}
	printf("%4.1f %13.9f %13.9f %13.9f %13.9f\n",
		x, creal(z1), cimag(z1), creal(z2), cimag(z2));
  }
  return 0;
}

// bern(x) + j*bein(x) = i^(-n)*Kn(x*sqrt(-1))
// bern'(x) = (d/dx)bern(x)

int bernbein()
{
  int i, n;
  double x;
  COMPLEX j, z, z1, z2;

  for (n = 1; n <= 5; n++) {
	if (n > 1)
		printf("\n");
	printf("   x      ber%d(x)      bei%d(x)     ber'%d(x)     bei'%d(x)\n",
		n, n, n, n);
	for (x = 1.0; x <= 10.0; x += 1.0) {
		z = csqrt(I) * x;
		z1 = I2N(n) * In((double)n, z);
		z2 = I2N(n) * csqrt(I) * Ind((double)n, z);
		printf("%4.1f %12.7f %12.7f %12.7f %12.7f\n",
			x, creal(z1), cimag(z1), creal(z2), cimag(z2));
	}
  }
  return 0;
}

// kern(x) + j*kein(x) = i^(-n)*Kn(x*sqrt(-1))
// kern'(x) = (d/dx)kern(x)

int kernkein()
{
  int n;
  double x;
  COMPLEX j, z, z1, z2;

  for (n = 1; n <= 5; n++) {
	if (n > 1)
		printf("\n");
	printf("   x       ker%d(x)       kei%d(x)      ker'%d(x)      kei'%d(x)\n",
		n, n, n, n);
	for (x = 1.0; x <= 10.0; x += 1.0) {
		z = csqrt(I) * x;
		z1 = I2N(-n) * Kn((double)n, z);
		z2 = I2N(-n) * csqrt(I) * Knd((double)n, z);
		printf("%4.1f %13.7f %13.7f %13.7f %13.7f\n",
			x, creal(z1), cimag(z1), creal(z2), cimag(z2));
	}
  }
  return 0;
}

// I to the n'th power  (n = .. -2, -2, 0, 1, 2, ..)

COMPLEX I2N(n) int n;
{
  static COMPLEX z[] = { 1.0, I, -1.0, -I };

  n %= 4;
  if (n < 0)
	n += 4;
  return z[n];
}
